@@ -17,7 +17,6 @@ Pluggable Back-end for Account Server
 """
 
 from uuid import uuid4
-import time
 import six.moves.cPickle as pickle
 
 import sqlite3
@@ -154,7 +153,7 @@ class AccountBroker(DatabaseBroker):
         conn.execute('''
             UPDATE account_stat SET account = ?, created_at = ?, id = ?,
                    put_timestamp = ?, status_changed_at = ?
-            ''', (self.account, Timestamp(time.time()).internal, str(uuid4()),
+            ''', (self.account, Timestamp.now().internal, str(uuid4()),
                   put_timestamp, put_timestamp))
 
     def create_policy_stat_table(self, conn):
