@@ -26,7 +26,6 @@ BufferedHTTPResponse.
     make all calls through httplib.
 """
 
-from swift import gettext_ as _
 from swift.common import constraints
 import logging
 import time
@@ -38,7 +37,10 @@ from eventlet.green.httplib import CONTINUE, HTTPConnection, HTTPMessage, \
 from six.moves.urllib.parse import quote
 import six
 
-httplib = eventlet.import_patched('httplib')
+if six.PY2:
+    httplib = eventlet.import_patched('httplib')
+else:
+    httplib = eventlet.import_patched('http.client')
 httplib._MAXHEADERS = constraints.MAX_HEADER_COUNT
 
 
@@ -200,15 +202,9 @@ def http_connect(ipaddr, port, device, partition, method, path,
     :returns: HTTPConnection object
     """
     if isinstance(path, six.text_type):
-        try:
-            path = path.encode("utf-8")
-        except UnicodeError as e:
-            logging.exception(_('Error encoding to UTF-8: %s'), str(e))
+        path = path.encode("utf-8")
     if isinstance(device, six.text_type):
-        try:
-            device = device.encode("utf-8")
-        except UnicodeError as e:
-            logging.exception(_('Error encoding to UTF-8: %s'), str(e))
+        device = device.encode("utf-8")
     path = quote('/' + device + '/' + str(partition) + path)
     return http_connect_raw(
         ipaddr, port, method, path, headers, query_string, ssl)
