@@ -21,7 +21,6 @@ through the proxy.
 import json
 import os
 import socket
-from time import time
 
 from eventlet import sleep, Timeout
 import six
@@ -43,6 +42,8 @@ class DirectClientException(ClientException):
         # host can be used to override the node ip and port reported in
         # the exception
         host = host if host is not None else node
+        if not isinstance(path, six.text_type):
+            path = path.decode("utf-8")
         full_path = quote('/%s/%s%s' % (node['device'], part, path))
         msg = '%s server %s:%s direct %s %r gave status %s' % (
             stype, host['ip'], host['port'], method, full_path, resp.status)
@@ -79,22 +80,28 @@ def _make_req(node, part, method, path, _headers, stype,
 
 def _get_direct_account_container(path, stype, node, part,
                                   marker=None, limit=None,
-                                  prefix=None, delimiter=None, conn_timeout=5,
-                                  response_timeout=15):
+                                  prefix=None, delimiter=None,
+                                  conn_timeout=5, response_timeout=15,
+                                  end_marker=None, reverse=None):
     """Base class for get direct account and container.
 
     Do not use directly use the get_direct_account or
     get_direct_container instead.
     """
-    qs = 'format=json'
+    params = ['format=json']
     if marker:
-        qs += '&marker=%s' % quote(marker)
+        params.append('marker=%s' % quote(marker))
     if limit:
-        qs += '&limit=%d' % limit
+        params.append('limit=%d' % limit)
     if prefix:
-        qs += '&prefix=%s' % quote(prefix)
+        params.append('prefix=%s' % quote(prefix))
     if delimiter:
-        qs += '&delimiter=%s' % quote(delimiter)
+        params.append('delimiter=%s' % quote(delimiter))
+    if end_marker:
+        params.append('end_marker=%s' % quote(end_marker))
+    if reverse:
+        params.append('reverse=%s' % quote(reverse))
+    qs = '&'.join(params)
     with Timeout(conn_timeout):
         conn = http_connect(node['ip'], node['port'], node['device'], part,
                             'GET', path, query_string=qs,
@@ -117,14 +124,14 @@ def _get_direct_account_container(path, stype, node, part,
 def gen_headers(hdrs_in=None, add_ts=False):
     hdrs_out = HeaderKeyDict(hdrs_in) if hdrs_in else HeaderKeyDict()
     if add_ts:
-        hdrs_out['X-Timestamp'] = Timestamp(time()).internal
+        hdrs_out['X-Timestamp'] = Timestamp.now().internal
     hdrs_out['User-Agent'] = 'direct-client %s' % os.getpid()
     return hdrs_out
 
 
 def direct_get_account(node, part, account, marker=None, limit=None,
                        prefix=None, delimiter=None, conn_timeout=5,
-                       response_timeout=15):
+                       response_timeout=15, end_marker=None, reverse=None):
     """
     Get listings directly from the account server.
 
@@ -137,6 +144,8 @@ def direct_get_account(node, part, account, marker=None, limit=None,
     :param delimiter: delimiter for the query
     :param conn_timeout: timeout in seconds for establishing the connection
     :param response_timeout: timeout in seconds for getting the response
+    :param end_marker: end_marker query
+    :param reverse: reverse the returned listing
     :returns: a tuple of (response headers, a list of containers) The response
               headers will HeaderKeyDict.
     """
@@ -145,6 +154,8 @@ def direct_get_account(node, part, account, marker=None, limit=None,
                                          marker=marker,
                                          limit=limit, prefix=prefix,
                                          delimiter=delimiter,
+                                         end_marker=end_marker,
+                                         reverse=reverse,
                                          conn_timeout=conn_timeout,
                                          response_timeout=response_timeout)
 
@@ -185,7 +196,8 @@ def direct_head_container(node, part, account, container, conn_timeout=5,
 
 def direct_get_container(node, part, account, container, marker=None,
                          limit=None, prefix=None, delimiter=None,
-                         conn_timeout=5, response_timeout=15):
+                         conn_timeout=5, response_timeout=15, end_marker=None,
+                         reverse=None):
     """
     Get container listings directly from the container server.
 
@@ -199,6 +211,8 @@ def direct_get_container(node, part, account, container, marker=None,
     :param delimiter: delimiter for the query
     :param conn_timeout: timeout in seconds for establishing the connection
     :param response_timeout: timeout in seconds for getting the response
+    :param end_marker: end_marker query
+    :param reverse: reverse the returned listing
     :returns: a tuple of (response headers, a list of objects) The response
               headers will be a HeaderKeyDict.
     """
@@ -207,6 +221,8 @@ def direct_get_container(node, part, account, container, marker=None,
                                          part, marker=marker,
                                          limit=limit, prefix=prefix,
                                          delimiter=delimiter,
+                                         end_marker=end_marker,
+                                         reverse=reverse,
                                          conn_timeout=conn_timeout,
                                          response_timeout=response_timeout)
 
