@@ -22,14 +22,14 @@ from six.moves import urllib
 import struct
 from sys import exc_info, exit
 import zlib
-from swift import gettext_ as _
 from time import gmtime, strftime, time
 from zlib import compressobj
 
 from swift.common.exceptions import ClientException
-from swift.common.http import HTTP_NOT_FOUND, HTTP_MULTIPLE_CHOICES
+from swift.common.http import (HTTP_NOT_FOUND, HTTP_MULTIPLE_CHOICES,
+                               is_server_error)
 from swift.common.swob import Request
-from swift.common.utils import quote
+from swift.common.utils import quote, closing_if_possible
 from swift.common.wsgi import loadapp, pipeline_property
 
 if six.PY3:
@@ -157,7 +157,8 @@ class InternalClient(object):
         'auto_create_account_prefix', default='.')
 
     def make_request(
-            self, method, path, headers, acceptable_statuses, body_file=None):
+            self, method, path, headers, acceptable_statuses, body_file=None,
+            params=None):
         """Makes a request to Swift with retries.
 
         :param method: HTTP method of request.
@@ -166,6 +167,8 @@ class InternalClient(object):
         :param acceptable_statuses: List of acceptable statuses for request.
         :param body_file: Body file to be passed along with request,
                           defaults to None.
+        :param params: A dict of params to be set in request query string,
+                       defaults to None.
 
         :returns: Response object on success.
 
@@ -177,27 +180,45 @@ class InternalClient(object):
 
         headers = dict(headers)
         headers['user-agent'] = self.user_agent
-        resp = exc_type = exc_value = exc_traceback = None
         for attempt in range(self.request_tries):
+            resp = exc_type = exc_value = exc_traceback = None
             req = Request.blank(
                 path, environ={'REQUEST_METHOD': method}, headers=headers)
             if body_file is not None:
                 if hasattr(body_file, 'seek'):
                     body_file.seek(0)
                 req.body_file = body_file
+            if params:
+                req.params = params
             try:
                 resp = req.get_response(self.app)
+            except (Exception, Timeout):
+                exc_type, exc_value, exc_traceback = exc_info()
+            else:
                 if resp.status_int in acceptable_statuses or \
                         resp.status_int // 100 in acceptable_statuses:
                     return resp
-            except (Exception, Timeout):
-                exc_type, exc_value, exc_traceback = exc_info()
+                elif not is_server_error(resp.status_int):
+                    # No sense retrying when we expect the same result
+                    break
             # sleep only between tries, not after each one
             if attempt < self.request_tries - 1:
+                if resp:
+                    # always close any resp.app_iter before we discard it
+                    with closing_if_possible(resp.app_iter):
+                        # for non 2XX requests it's safe and useful to drain
+                        # the response body so we log the correct status code
+                        if resp.status_int // 100 != 2:
+                            for iter_body in resp.app_iter:
+                                pass
                 sleep(2 ** (attempt + 1))
         if resp:
-            raise UnexpectedResponse(
-                _('Unexpected response: %s') % resp.status, resp)
+            msg = 'Unexpected response: %s' % resp.status
+            if resp.status_int // 100 != 2 and resp.body:
+                # provide additional context (and drain the response body) for
+                # non 2XX responses
+                msg += ' (%s)' % resp.body
+            raise UnexpectedResponse(msg, resp)
         if exc_type:
             # To make pep8 tool happy, in place of raise t, v, tb:
             six.reraise(exc_type(*exc_value.args), None, exc_traceback)
@@ -236,7 +257,7 @@ class InternalClient(object):
         return metadata
 
     def _iter_items(
-            self, path, marker='', end_marker='',
+            self, path, marker='', end_marker='', prefix='',
             acceptable_statuses=(2, HTTP_NOT_FOUND)):
         """
         Returns an iterator of items from a json listing.  Assumes listing has
@@ -246,6 +267,7 @@ class InternalClient(object):
         :param marker: Prefix of first desired item, defaults to ''.
         :param end_marker: Last item returned will be 'less' than this,
                            defaults to ''.
+        :param prefix: Prefix of items
         :param acceptable_statuses: List of status for valid responses,
                                     defaults to (2, HTTP_NOT_FOUND).
 
@@ -257,8 +279,8 @@ class InternalClient(object):
 
         while True:
             resp = self.make_request(
-                'GET', '%s?format=json&marker=%s&end_marker=%s' %
-                (path, quote(marker), quote(end_marker)),
+                'GET', '%s?format=json&marker=%s&end_marker=%s&prefix=%s' %
+                (path, quote(marker), quote(end_marker), quote(prefix)),
                 {}, acceptable_statuses)
             if not resp.status_int == 200:
                 if resp.status_int >= HTTP_MULTIPLE_CHOICES:
@@ -326,7 +348,7 @@ class InternalClient(object):
     # account methods
 
     def iter_containers(
-            self, account, marker='', end_marker='',
+            self, account, marker='', end_marker='', prefix='',
             acceptable_statuses=(2, HTTP_NOT_FOUND)):
         """
         Returns an iterator of containers dicts from an account.
@@ -335,6 +357,7 @@ class InternalClient(object):
         :param marker: Prefix of first desired item, defaults to ''.
         :param end_marker: Last item returned will be 'less' than this,
                            defaults to ''.
+        :param prefix: Prefix of containers
         :param acceptable_statuses: List of status for valid responses,
                                     defaults to (2, HTTP_NOT_FOUND).
 
@@ -345,7 +368,8 @@ class InternalClient(object):
         """
 
         path = self.make_path(account)
-        return self._iter_items(path, marker, end_marker, acceptable_statuses)
+        return self._iter_items(path, marker, end_marker, prefix,
+                                acceptable_statuses)
 
     def get_account_info(
             self, account, acceptable_statuses=(2, HTTP_NOT_FOUND)):
@@ -503,7 +527,7 @@ class InternalClient(object):
         return self._get_metadata(path, metadata_prefix, acceptable_statuses)
 
     def iter_objects(
-            self, account, container, marker='', end_marker='',
+            self, account, container, marker='', end_marker='', prefix='',
             acceptable_statuses=(2, HTTP_NOT_FOUND)):
         """
         Returns an iterator of object dicts from a container.
@@ -513,6 +537,7 @@ class InternalClient(object):
         :param marker: Prefix of first desired item, defaults to ''.
         :param end_marker: Last item returned will be 'less' than this,
                            defaults to ''.
+        :param prefix: Prefix of objects
         :param acceptable_statuses: List of status for valid responses,
                                     defaults to (2, HTTP_NOT_FOUND).
 
@@ -523,7 +548,8 @@ class InternalClient(object):
         """
 
         path = self.make_path(account, container)
-        return self._iter_items(path, marker, end_marker, acceptable_statuses)
+        return self._iter_items(path, marker, end_marker, prefix,
+                                acceptable_statuses)
 
     def set_container_metadata(
             self, account, container, metadata, metadata_prefix='',
@@ -606,14 +632,30 @@ class InternalClient(object):
                                   headers=headers)
 
     def get_object(self, account, container, obj, headers,
-                   acceptable_statuses=(2,)):
+                   acceptable_statuses=(2,), params=None):
         """
-        Returns a 3-tuple (status, headers, iterator of object body)
+        Gets an object.
+
+        :param account: The object's account.
+        :param container: The object's container.
+        :param obj: The object name.
+        :param headers: Headers to send with request, defaults to empty dict.
+        :param acceptable_statuses: List of status for valid responses,
+                                    defaults to (2,).
+        :param params: A dict of params to be set in request query string,
+                       defaults to None.
+
+        :raises UnexpectedResponse: Exception raised when requests fail
+                                    to get a response with an acceptable status
+        :raises Exception: Exception is raised when code fails in an
+                           unexpected way.
+        :returns: A 3-tuple (status, headers, iterator of object body)
         """
 
         headers = headers or {}
         path = self.make_path(account, container, obj)
-        resp = self.make_request('GET', path, headers, acceptable_statuses)
+        resp = self.make_request(
+            'GET', path, headers, acceptable_statuses, params=params)
         return (resp.status_int, resp.headers, resp.app_iter)
 
     def iter_object_lines(
@@ -697,7 +739,7 @@ class InternalClient(object):
         :param account: The object's account.
         :param container: The object's container.
         :param obj: The object.
-        :param headers: Headers to send with request, defaults ot empty dict.
+        :param headers: Headers to send with request, defaults to empty dict.
 
         :raises UnexpectedResponse: Exception raised when requests fail
                                     to get a response with an acceptable status
