@@ -34,7 +34,8 @@ from swift.common.storage_policy import POLICIES
 from swift.common.exceptions import ListingIterError, SegmentError
 from swift.common.http import is_success
 from swift.common.swob import HTTPBadRequest, \
-    HTTPServiceUnavailable, Range, is_chunked, multi_range_iterator
+    HTTPServiceUnavailable, Range, is_chunked, multi_range_iterator, \
+    HTTPPreconditionFailed
 from swift.common.utils import split_path, validate_device_partition, \
     close_if_possible, maybe_multipart_byteranges_to_document_iters, \
     multipart_byteranges_to_document_iters, parse_content_type, \
@@ -281,6 +282,31 @@ def copy_header_subset(from_r, to_r, condition):
             to_r.headers[k] = v
 
 
+def check_path_header(req, name, length, error_msg):
+    """
+    Validate that the value of path-like header is
+    well formatted. We assume the caller ensures that
+    specific header is present in req.headers.
+
+    :param req: HTTP request object
+    :param name: header name
+    :param length: length of path segment check
+    :param error_msg: error message for client
+    :returns: A tuple with path parts according to length
+    :raise: HTTPPreconditionFailed if header value
+            is not well formatted.
+    """
+    hdr = unquote(req.headers.get(name))
+    if not hdr.startswith('/'):
+        hdr = '/' + hdr
+    try:
+        return split_path(hdr, length, length, True)
+    except ValueError:
+        raise HTTPPreconditionFailed(
+            request=req,
+            body=error_msg)
+
+
 class SegmentedIterable(object):
     """
     Iterable that returns the object contents for a large object.
@@ -328,12 +354,25 @@ class SegmentedIterable(object):
 
     def _coalesce_requests(self):
         start_time = time.time()
-        pending_req = None
-        pending_etag = None
-        pending_size = None
+        pending_req = pending_etag = pending_size = None
         try:
-            for seg_path, seg_etag, seg_size, first_byte, last_byte \
-                    in self.listing_iter:
+            for seg_dict in self.listing_iter:
+                if 'raw_data' in seg_dict:
+                    if pending_req:
+                        yield pending_req, pending_etag, pending_size
+
+                    to_yield = seg_dict['raw_data'][
+                        seg_dict['first_byte']:seg_dict['last_byte'] + 1]
+                    yield to_yield, None, len(seg_dict['raw_data'])
+                    pending_req = pending_etag = pending_size = None
+                    continue
+
+                seg_path, seg_etag, seg_size, first_byte, last_byte = (
+                    seg_dict['path'], seg_dict.get('hash'),
+                    seg_dict.get('bytes'),
+                    seg_dict['first_byte'], seg_dict['last_byte'])
+                if seg_size is not None:
+                    seg_size = int(seg_size)
                 first_byte = first_byte or 0
                 go_to_end = last_byte is None or (
                     seg_size is not None and last_byte == seg_size - 1)
@@ -415,7 +454,18 @@ class SegmentedIterable(object):
         bytes_left = self.response_body_length
 
         try:
-            for seg_req, seg_etag, seg_size in self._coalesce_requests():
+            for data_or_req, seg_etag, seg_size in self._coalesce_requests():
+                if isinstance(data_or_req, bytes):
+                    chunk = data_or_req  # ugly, awful overloading
+                    if bytes_left is None:
+                        yield chunk
+                    elif bytes_left >= len(chunk):
+                        yield chunk
+                        bytes_left -= len(chunk)
+                    else:
+                        yield chunk[:bytes_left]
+                    continue
+                seg_req = data_or_req
                 seg_resp = seg_req.get_response(self.app)
                 if not is_success(seg_resp.status_int):
                     close_if_possible(seg_resp.app_iter)
@@ -645,7 +695,7 @@ def resolve_etag_is_at_header(req, metadata):
     middleware's alternate etag sysmeta (X-Object-Sysmeta-Crypto-Etag) but will
     then find the EC alternate etag (if EC policy). But if the object *is*
     encrypted then X-Object-Sysmeta-Crypto-Etag is found and used, which is
-    correct because it should be preferred over X-Object-Sysmeta-Crypto-Etag.
+    correct because it should be preferred over X-Object-Sysmeta-Ec-Etag.
 
     :param req: a swob Request
     :param metadata: a dict containing object metadata
