@@ -202,7 +202,6 @@ class Connection(object):
 
     def http_connect(self):
         self.connection = self.conn_class(self.storage_netloc)
-        # self.connection.set_debuglevel(3)
 
     def make_path(self, path=None, cfg=None):
         if path is None:
@@ -330,7 +329,6 @@ class Connection(object):
             path = '%s?%s' % (path, '&'.join(query_args))
 
         self.connection = self.conn_class(self.storage_netloc)
-        # self.connection.set_debuglevel(3)
         self.connection.putrequest('PUT', path)
         for key, value in headers.items():
             self.connection.putheader(key, value)
@@ -796,7 +794,8 @@ class File(Base):
                   ['content_type', 'content-type'],
                   ['last_modified', 'last-modified'],
                   ['etag', 'etag']]
-        optional_fields = [['x_object_manifest', 'x-object-manifest']]
+        optional_fields = [['x_object_manifest', 'x-object-manifest'],
+                           ['x_symlink_target', 'x-symlink-target']]
 
         header_fields = self.header_fields(fields,
                                            optional_fields=optional_fields)
