@@ -18,14 +18,13 @@
 import datetime
 import json
 import unittest2
-from unittest2 import SkipTest
 from uuid import uuid4
 import time
 
 from six.moves import range
 
 from test.functional import check_response, retry, requires_acls, \
-    requires_policies
+    requires_policies, SkipTest
 import test.functional as tf
 
 
@@ -330,8 +329,10 @@ class TestObject(unittest2.TestCase):
                               'Content-Length': '0'})
             return check_response(conn)
         ts_before = time.time()
+        time.sleep(0.05)
         resp = retry(put)
         body = resp.read()
+        time.sleep(0.05)
         ts_after = time.time()
         if resp.status == 400:
             # shunt_inbound_x_timestamp must be false
@@ -361,8 +362,10 @@ class TestObject(unittest2.TestCase):
                               'Content-Length': '0'})
             return check_response(conn)
         ts_before = time.time()
+        time.sleep(0.05)
         resp = retry(put)
         body = resp.read()
+        time.sleep(0.05)
         ts_after = time.time()
         if resp.status == 400:
             # shunt_inbound_x_timestamp must be false
