@@ -73,6 +73,13 @@ class TestEmptyDevice(ReplProbeTest):
             raise Exception('Object GET did not return VERIFY, instead it '
                             'returned: %s' % repr(odata))
 
+        # Stash the on disk data from a primary for future comparison with the
+        # handoff - this may not equal 'VERIFY' if for example the proxy has
+        # crypto enabled
+        direct_get_data = direct_client.direct_get_object(
+            onodes[1], opart, self.account, container, obj, headers={
+                'X-Backend-Storage-Policy-Index': self.policy.idx})[-1]
+
         # Kill other two container/obj primary servers
         #  to ensure GET handoff works
         for node in onodes[1:]:
@@ -95,9 +102,7 @@ class TestEmptyDevice(ReplProbeTest):
         odata = direct_client.direct_get_object(
             another_onode, opart, self.account, container, obj,
             headers={'X-Backend-Storage-Policy-Index': self.policy.idx})[-1]
-        if odata != 'VERIFY':
-            raise Exception('Direct object GET did not return VERIFY, instead '
-                            'it returned: %s' % repr(odata))
+        self.assertEqual(direct_get_data, odata)
 
         # Assert container listing (via proxy and directly) has container/obj
         objs = [o['name'] for o in
@@ -155,9 +160,7 @@ class TestEmptyDevice(ReplProbeTest):
         odata = direct_client.direct_get_object(
             onode, opart, self.account, container, obj, headers={
                 'X-Backend-Storage-Policy-Index': self.policy.idx})[-1]
-        if odata != 'VERIFY':
-            raise Exception('Direct object GET did not return VERIFY, instead '
-                            'it returned: %s' % repr(odata))
+        self.assertEqual(direct_get_data, odata)
 
         # Assert the handoff server no longer has container/obj
         try:
