@@ -17,19 +17,23 @@
 """Tests for swift.common.utils"""
 
 from collections import defaultdict
+import errno
 from hashlib import md5
-import logging
+import six
 import socket
 import time
 import unittest
 from uuid import uuid4
+import os
+
+import mock
 
 from eventlet import GreenPool, sleep, Queue
 from eventlet.pools import Pool
 
 from swift.common import memcached
 from mock import patch, MagicMock
-from test.unit import NullLoggingHandler
+from test.unit import debug_logger
 
 
 class MockedMemcachePool(memcached.MemcacheConnPool):
@@ -48,15 +52,15 @@ class ExplodingMockMemcached(object):
 
     def sendall(self, string):
         self.exploded = True
-        raise socket.error()
+        raise socket.error(errno.EPIPE, os.strerror(errno.EPIPE))
 
     def readline(self):
         self.exploded = True
-        raise socket.error()
+        raise socket.error(errno.EPIPE, os.strerror(errno.EPIPE))
 
     def read(self, size):
         self.exploded = True
-        raise socket.error()
+        raise socket.error(errno.EPIPE, os.strerror(errno.EPIPE))
 
     def close(self):
         pass
@@ -73,6 +77,7 @@ class MockMemcached(object):
         self.down = False
         self.exc_on_delete = False
         self.read_return_none = False
+        self.read_return_empty_str = False
         self.close_called = False
 
     def sendall(self, string):
@@ -145,6 +150,8 @@ class MockMemcached(object):
             self.outbuf += 'NOT_FOUND\r\n'
 
     def readline(self):
+        if self.read_return_empty_str:
+            return ''
         if self.read_return_none:
             return None
         if self.down:
@@ -169,6 +176,12 @@ class MockMemcached(object):
 class TestMemcached(unittest.TestCase):
     """Tests for swift.common.memcached"""
 
+    def setUp(self):
+        self.logger = debug_logger()
+        patcher = mock.patch('swift.common.memcached.logging', self.logger)
+        self.addCleanup(patcher.stop)
+        patcher.start()
+
     def test_get_conns(self):
         sock1 = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         sock1.bind(('127.0.0.1', 0))
@@ -327,6 +340,31 @@ class TestMemcached(unittest.TestCase):
         _junk, cache_timeout, _junk = mock.cache[cache_key]
         self.assertAlmostEqual(float(cache_timeout), esttimeout, delta=1)
 
+    def test_get_failed_connection_mid_request(self):
+        memcache_client = memcached.MemcacheRing(['1.2.3.4:11211'])
+        mock = MockMemcached()
+        memcache_client._client_cache['1.2.3.4:11211'] = MockedMemcachePool(
+            [(mock, mock)] * 2)
+        memcache_client.set('some_key', [1, 2, 3])
+        self.assertEqual(memcache_client.get('some_key'), [1, 2, 3])
+        self.assertEqual(mock.cache.values()[0][1], '0')
+
+        # Now lets return an empty string, and make sure we aren't logging
+        # the error.
+        fake_stdout = six.StringIO()
+        # force the logging through the DebugLogger instead of the nose
+        # handler. This will use stdout, so we can assert that no stack trace
+        # is logged.
+        logger = debug_logger()
+        with patch("sys.stdout", fake_stdout),\
+                patch('swift.common.memcached.logging', logger):
+            mock.read_return_empty_str = True
+            self.assertEqual(memcache_client.get('some_key'), None)
+        log_lines = logger.get_lines_for_level('error')
+        self.assertIn('Error talking to memcached', log_lines[0])
+        self.assertFalse(log_lines[1:])
+        self.assertNotIn("Traceback", fake_stdout.getvalue())
+
     def test_incr(self):
         memcache_client = memcached.MemcacheRing(['1.2.3.4:11211'])
         mock = MockMemcached()
@@ -347,6 +385,33 @@ class TestMemcached(unittest.TestCase):
                           memcache_client.incr, 'some_key', delta=-15)
         self.assertTrue(mock.close_called)
 
+    def test_incr_failed_connection_mid_request(self):
+        memcache_client = memcached.MemcacheRing(['1.2.3.4:11211'])
+        mock = MockMemcached()
+        memcache_client._client_cache['1.2.3.4:11211'] = MockedMemcachePool(
+            [(mock, mock)] * 2)
+        self.assertEqual(memcache_client.incr('some_key', delta=5), 5)
+        self.assertEqual(memcache_client.get('some_key'), '5')
+        self.assertEqual(memcache_client.incr('some_key', delta=5), 10)
+        self.assertEqual(memcache_client.get('some_key'), '10')
+
+        # Now lets return an empty string, and make sure we aren't logging
+        # the error.
+        fake_stdout = six.StringIO()
+        # force the logging through the DebugLogger instead of the nose
+        # handler. This will use stdout, so we can assert that no stack trace
+        # is logged.
+        logger = debug_logger()
+        with patch("sys.stdout", fake_stdout), \
+                patch('swift.common.memcached.logging', logger):
+            mock.read_return_empty_str = True
+            self.assertRaises(memcached.MemcacheConnectionError,
+                              memcache_client.incr, 'some_key', delta=1)
+        log_lines = logger.get_lines_for_level('error')
+        self.assertIn('Error talking to memcached', log_lines[0])
+        self.assertFalse(log_lines[1:])
+        self.assertNotIn('Traceback', fake_stdout.getvalue())
+
     def test_incr_w_timeout(self):
         memcache_client = memcached.MemcacheRing(['1.2.3.4:11211'])
         mock = MockMemcached()
@@ -397,7 +462,6 @@ class TestMemcached(unittest.TestCase):
                           memcache_client.decr, 'some_key', delta=15)
 
     def test_retry(self):
-        logging.getLogger().addHandler(NullLoggingHandler())
         memcache_client = memcached.MemcacheRing(
             ['1.2.3.4:11211', '1.2.3.5:11211'])
         mock1 = ExplodingMockMemcached()
@@ -405,10 +469,25 @@ class TestMemcached(unittest.TestCase):
         memcache_client._client_cache['1.2.3.4:11211'] = MockedMemcachePool(
             [(mock2, mock2)])
         memcache_client._client_cache['1.2.3.5:11211'] = MockedMemcachePool(
-            [(mock1, mock1)])
+            [(mock1, mock1), (mock1, mock1)])
         memcache_client.set('some_key', [1, 2, 3])
+        self.assertEqual(mock1.exploded, True)
+        self.assertEqual(self.logger.get_lines_for_level('error'), [
+            'Error talking to memcached: 1.2.3.5:11211: '
+            '[Errno 32] Broken pipe',
+        ])
+
+        self.logger.clear()
+        mock1.exploded = False
         self.assertEqual(memcache_client.get('some_key'), [1, 2, 3])
         self.assertEqual(mock1.exploded, True)
+        self.assertEqual(self.logger.get_lines_for_level('error'), [
+            'Error talking to memcached: 1.2.3.5:11211: '
+            '[Errno 32] Broken pipe',
+        ])
+        # Check that we really did call create() twice
+        self.assertEqual(memcache_client._client_cache['1.2.3.5:11211'].mocks,
+                         [])
 
     def test_delete(self):
         memcache_client = memcached.MemcacheRing(['1.2.3.4:11211'])
@@ -458,6 +537,17 @@ class TestMemcached(unittest.TestCase):
             ('some_key2', 'some_key1', 'not_exists'), 'multi_key'),
             [[4, 5, 6], [1, 2, 3], None])
 
+        # Now lets simulate a lost connection and make sure we don't get
+        # the index out of range stack trace when it does
+        mock_stderr = six.StringIO()
+        not_expected = "IndexError: list index out of range"
+        with patch("sys.stderr", mock_stderr):
+            mock.read_return_empty_str = True
+            self.assertEqual(memcache_client.get_multi(
+                ('some_key2', 'some_key1', 'not_exists'), 'multi_key'),
+                None)
+            self.assertFalse(not_expected in mock_stderr.getvalue())
+
     def test_serialization(self):
         memcache_client = memcached.MemcacheRing(['1.2.3.4:11211'],
                                                  allow_pickle=True)
@@ -544,25 +634,23 @@ class TestMemcached(unittest.TestCase):
             self.assertTrue(connections.empty())
 
     def test_connection_pool_timeout(self):
-        orig_conn_pool = memcached.MemcacheConnPool
-        try:
-            connections = defaultdict(Queue)
-            pending = defaultdict(int)
-            served = defaultdict(int)
+        connections = defaultdict(Queue)
+        pending = defaultdict(int)
+        served = defaultdict(int)
 
-            class MockConnectionPool(orig_conn_pool):
-                def get(self):
-                    pending[self.host] += 1
-                    conn = connections[self.host].get()
-                    pending[self.host] -= 1
-                    return conn
+        class MockConnectionPool(memcached.MemcacheConnPool):
+            def get(self):
+                pending[self.host] += 1
+                conn = connections[self.host].get()
+                pending[self.host] -= 1
+                return conn
 
-                def put(self, *args, **kwargs):
-                    connections[self.host].put(*args, **kwargs)
-                    served[self.host] += 1
-
-            memcached.MemcacheConnPool = MockConnectionPool
+            def put(self, *args, **kwargs):
+                connections[self.host].put(*args, **kwargs)
+                served[self.host] += 1
 
+        with mock.patch.object(memcached, 'MemcacheConnPool',
+                               MockConnectionPool):
             memcache_client = memcached.MemcacheRing(['1.2.3.4:11211',
                                                       '1.2.3.5:11211'],
                                                      io_timeout=0.5,
@@ -587,18 +675,20 @@ class TestMemcached(unittest.TestCase):
             # Wait for the dust to settle.
             p.waitall()
 
-            self.assertEqual(pending['1.2.3.5'], 8)
-            self.assertEqual(len(memcache_client._errors['1.2.3.5:11211']), 8)
-            self.assertEqual(served['1.2.3.5'], 2)
-            self.assertEqual(pending['1.2.3.4'], 0)
-            self.assertEqual(len(memcache_client._errors['1.2.3.4:11211']), 0)
-            self.assertEqual(served['1.2.3.4'], 8)
+        self.assertEqual(pending['1.2.3.5'], 8)
+        self.assertEqual(len(memcache_client._errors['1.2.3.5:11211']), 8)
+        self.assertEqual(
+            self.logger.get_lines_for_level('error'),
+            ['Timeout getting a connection to memcached: 1.2.3.5:11211'] * 8)
+        self.assertEqual(served['1.2.3.5'], 2)
+        self.assertEqual(pending['1.2.3.4'], 0)
+        self.assertEqual(len(memcache_client._errors['1.2.3.4:11211']), 0)
+        self.assertEqual(served['1.2.3.4'], 8)
+
+        # and we never got more put in that we gave out
+        self.assertEqual(connections['1.2.3.5'].qsize(), 2)
+        self.assertEqual(connections['1.2.3.4'].qsize(), 2)
 
-            # and we never got more put in that we gave out
-            self.assertEqual(connections['1.2.3.5'].qsize(), 2)
-            self.assertEqual(connections['1.2.3.4'].qsize(), 2)
-        finally:
-            memcached.MemcacheConnPool = orig_conn_pool
 
 if __name__ == '__main__':
     unittest.main()
